<?php
/*
* @link http://www.kalcaddle.com/
* @author warlee | e-mail:kalcaddle@qq.com
* @copyright warlee 2014.(Shanghai)Co.,Ltd
* @license http://kalcaddle.com/tools/licenses/license.txt
*/

class app extends Controller{
    private static $core;

    function __construct(){
        parent::__construct();
        self::$core = new core();
        if(isset($this->in['id'])){
            $this->in['id'] = preg_replace("/[\/\s ]+/", "", $this->in['id']);
        }
        $this->check_application();
    }

    private function init_sql(){
        if(self::$url == null){
            $apphost = self::$core->sys['system']['apphost'];
            if(empty($apphost)) $apphost = "https://app.terra-master.com";
            $version = self::$core->_version();
            $kernel = self::$core->_kernel();
            $memlist = self::$core->_membysize('mem');
            $total = trim(rtrim($memlist['total'],"M"));
            $langType = LANGUAGE_TYPE;
            self::$url = "$apphost/api.43.php?lang={$langType}&v={$version}&m={$total}&k={$kernel}";
        }
        if(self::$netfile === null){
            self::$netfile = new netFile();
        }
    }

    /*用户首页展示*/
    public function index(){
        $this->display(TEMPLATE.'app/index.php');
    }

    # 检查 application
    private function check_application(){
        $LOCAL = "/usr/local";
        $REAL_LOCAL = @realpath($LOCAL);
        if(empty($REAL_LOCAL) || $REAL_LOCAL == $LOCAL){
            $REAL_PUBLIC = @realpath("/mnt/public");
            if(empty($REAL_PUBLIC)){
                show_json($this->L['error'], false);
            }
            $VOLUME = @dirname($REAL_PUBLIC);
            $APP = "$VOLUME/application";
            @system("rm -fr /usr/local >/dev/null;mkdir $APP >/dev/null;ln -fsn $APP $LOCAL");
        }
        if(!file_exists("$LOCAL/pkgs")){
            @mkdir("$LOCAL/pkgs", 0755);
        }
        if(!file_exists("$LOCAL/pkgs/upload")){
            @mkdir("$LOCAL/pkgs/upload", 0755);
        }
    }

    #桌面统计应用更新数量
    public function check(){
        $this->init_sql();
        $return['key'] = "market.png";
        $return['item'] = self::$netfile->getItemCount();
        show_json($this->L['success'], true, $return);
    }

    public function init_app($user_info){
        $this->init_sql();
        $desktop = USER_PATH.$user_info['name'].'/home/desktop/';
        if(!file_exists($desktop)) {
            $list = self::$netfile->get();
            $new_user_app = $this->config['setting_system']['new_user_app'];
            $default = explode(',',$new_user_app);
            $info = array();
            foreach ($default as $key) {
                $info[$key] = $list[$key];
            }
            mk_dir($desktop);
            foreach ($info as $key => $data) {
                if (!is_array($data)) continue;
                $path = iconv_system($desktop . $key . '.oexe');
                unset($data['name']);
                unset($data['desc']);
                unset($data['group']);
                file_put_contents($path, json_encode($data));
            }
        }
    }

    /**
     * 用户app 添加、编辑
     */
    public function user_app() {
        $this->init_sql();
        $id = $this->in['id'];
        $data = self::$netfile->get($id);
        if(empty($data)){
            show_json($this->L['error']);exit;
        }
        //app update
        if($this->in['action'] == 'update'){
            touch("/tmp/.remove.$id");
            self::$netfile->delete($id);
        }
        $res = self::$netfile->install($data);
        if($res === true){
            show_json($this->L['success'], true, $res);
        }else{
            show_json($this->L['loading'], true, $data['id']);
        }
    }

    private function load_config_ini($id){
        $config = "/usr/local/pkgs/{$id}/config.ini";
        $json = [];
        if(file_exists($config)){
            $json_string = @file_get_contents($config);
            if(!empty($json_string)){
                $json = @json_decode($json_string, true);
            }
        }
        return $json;
    }

    public function user_send(){
        $this->init_sql();
        $id = $this->in['id'];
        $json = $this->load_config_ini($id);
        $langfile = "/usr/local/pkgs/{$id}/${id}.lang";
        if(file_exists($langfile)) {
            $Lng = @parse_ini_file($langfile, true);
            $langType = LANGUAGE_TYPE;
            if(isset($Lng[$langType])){
                $json['name'] = $Lng[$langType]['name'];
            }else if(isset($Lng['en-us'])){
                $json['name'] = $Lng['en-us']['name'];
            }else{
                show_json($this->L['error'], false);
            }
        }
        if(count($json) < 2){
            $data = self::$netfile->get($id);
            unset($data['desc'], $data['group'], $data['thumbs'], $data['release_note']);
            $data['icon'] = self::$netfile->get_app_icon($data['id']);
            $json = array_merge($data, $json);
        }
        self::$netfile->send_desktop($json);
        show_json($this->L['success'], true);
    }

    #校验app安装包的正确性
    public function hand_app(){
        $this->init_sql();
        $package_name = $this->in['name'];
        $package = "/usr/local/pkgs/upload/$package_name";
        if(preg_match("/\.tpk$/", $package_name)){
            $json_string=$this->_exec("dd if=$package bs=1024 count=1 2>/dev/null");
            $json = @json_decode($json_string, true);
            if($json === false) show_json($this->L['error'], false);
            if($json['platform'] != self::$core->platform()){
                show_json($this->L['error'], false);
            }
            $langs = @shell_exec("dd if=$package bs=1024 skip=1 count=9 2>/dev/null");
            $L = @parse_ini_string($langs, true);
            if(is_array($L) && isset($L[LANGUAGE_TYPE])){
                $L[LANGUAGE_TYPE]['APPID'] = $json['id'];
                $L[LANGUAGE_TYPE]['version'] = $json['version'];
                show_json($this->L['success'], true, $L[LANGUAGE_TYPE]);
            }
        }
        show_json($this->L['error'], false);
    }

    public function hand_app_install(){
        $this->init_sql();
        $package_id = $this->in['id'];
        $package_name = $this->in['name'];
        $package = "/usr/local/pkgs/upload/$package_name";
        if(file_exists("/usr/local/uninstall/$package_id")){
            self::$netfile->delete($package_id);//清除旧的程序...
        }
        if(!file_exists($package)){
            show_json($this->L['error'], false);
        }
        $this->_backexec("{$GLOBALS['_appinstall']} install '$package_id' '{$package}' &");
        show_json($this->L['success'], true);
    }

    public function user_stat(){
        $this->init_sql();
        $id = $this->in['id'];// 例：$package_name
        $return['id'] = $id;
        $return['stat'] = $this->L['installing'];
        $wget_file = "/tmp/wget-{$id}";
        if(file_exists($wget_file)){
            $download = self::$netfile->_onlineprogress($wget_file);
            if($download == 400){
                show_json($this->L['error'], false, $return);
            }else if($download != 100){
                $return['process'] = ($download/2)."%";
                show_json($this->L['success'], true, $return);
            }
        }
        $download = 50;
        $json_file = "/tmp/databack/json-{$id}";
        if(!file_exists($json_file)){
            $return['process'] = ($download)."%";
            show_json($this->L['success'], true, $return);
        }else{
            $result = json_decode(file_get_contents($json_file), true);
            $inpg = round($result['now'] * 50 / $result['total'], 2);
            if($inpg > 50) $inpg = 50;
            if(($download + $inpg) >= 100){
                self::$netfile->setItem($id, "install", 1);
                $this->appAddConfig($id);
                $json = $this->load_config_ini($id);
                $return = array_merge($return, $json);
            }
            $return['process'] = ($download + $inpg)."%";
            if(!empty($return['error'])){
                show_json($this->L['error'], false, $return);
            }
            show_json($this->L['success'], true, $return);
        }
    }

    //  增加已安装的应用的信息
    private function appAddConfig($id){
        $info = $this->_exec("cat /usr/local/pkgs/{$id}/config.ini");
        $json = @json_decode($info, true);
        if($json){
            $return['exec'] = $json["exec"];
        }
        $data = self::$netfile->getItem($id);
        if(count($data) > 0){
            if(!empty($data['category'])){
                $type = $data['category'];
                $data['category_name'] = $GLOBALS['root']['app'][$type];
            }
            $da = @json_encode($data);
        }else{
            $appInfo = @parse_ini_file("/usr/local/pkgs/{$id}/{$id}.lang",true);
            $json["install"] = 1;
            $json["type"] = "app";
            $json["category"] = 1;
            $json['category_name'] = $GLOBALS['root']['app']["Utilities"];
            $json["name"] = $appInfo[LANGUAGE_TYPE]["name"];
            $da = @json_encode($json);
        }
        $path = "/etc/store/installed/{$id}";
        @touch($path);
        $this->_exec("echo '{$da}'> {$path}");
    }

    /**
     * 获取列表
     */
    public function get() {
        $this->init_sql();
        $appSafetyMod = $this->_exec("cat /etc/store/forbidden_list.conf", "array");
        $absoluteSafe = $this->_exec("cat /etc/firewalld/absolute_safe.conf");
        $absoluteSafe = explode("=", $absoluteSafe);
        if (!isset($this->in['group']) || $this->in['group']=='all') {
            $list = self::$netfile->get();
        }else{
            $list = self::$netfile->get('group','',$this->in['group']);
        }
        $list = array_reverse($list);
        $listFile = array();
        foreach ($list as $id => &$item){
            if($absoluteSafe[1] == 1 && is_array($appSafetyMod) && in_array($item["id"], $appSafetyMod)) continue;
            if(!empty($item['category'])){
                $type = $item['category'];
                $item['category_name'] = $GLOBALS['root']['app'][$type];
            }
            $listFile[$id] = $item;
        }
        show_json($listFile);
    }

    /**
     * 添加
     */
    public function add() {
        $this->init_sql();
        $res=self::$netfile->add(rawurldecode($this->in['name']), $this->_init());
        if($res) show_json($this->L['success']);
        show_json($this->L['error_repeat'],false);
    }

    /**
     * 删除
     */
    public function del() {
        $this->init_sql();
        $id = $this->in['id'];
        $name = $this->in['name'];
        @system("rm -f ".USER."home/desktop/{$name}.*.oexe");

        $result = self::$netfile->delete($id);
        if($result == 1){
            $path = "/etc/store/installed/{$id}";
            if(file_exists($path)){
                @unlink($path);
            }
            show_json($this->L['success']);
        }else{
            show_json($this->L['error'],false);
        }
    }

    public function del_stat(){
        $this->init_sql();
        $id = $this->in['id'];
        if(file_exists("/usr/local/uninstall/{$id}")){
            show_json($this->L['app_uninstalling'], false);
        }
        self::$netfile->setItem($id, "install", 0);
        sleep(1);
        show_json($this->L['success']);
    }

    /**
     * 应用状态
     * */
    public function app_status(){
        set_time_limit(0);
        $id = $this->in['id'];
        $this->init_sql();
        show_json(self::$netfile->get_app_status($id));
    }

    /**
     * 应用启动关闭
     * */
    public function app_start_stop(){
        set_time_limit(0);
        $this->init_sql();
        $id = $this->in['id']; //不能包含[ /]
        $start = $this->in['start'];
        $path = "/etc/init.d/{$id}";
        if(file_exists($path)){
            if($start){
                @system("{$path} start >/dev/null");
                $keystring = "running";
            }else{
                @system("{$path} stop >/dev/null");
                $keystring = "stopped";
                $name = addcslashes($this->in['name'], ' \'"');
                @system("rm -f ".USER."home/desktop/{$name}.*.oexe");
            }
            sleep(3);
            $result = $this->_exec("{$path} status");
            $status = strstr($result, $keystring) !==false ? "1" : "2";
            $sc = "/etc/sc.d/{$id}";
            if($status == 1){
                @touch($sc);
            }else{
                @unlink($sc);
            }
        }else{
            $status = "0";
        }
        show_json($status);
    }

    /**
     * 应用自动更新
     * */
    public function get_auto_update(){
        $setPath = "/etc/store/app.conf";
        $file_dir = dirname($setPath);
        if (!file_exists($file_dir)) {
            @system("mkdir -p {$file_dir}");
            @system("chmod -R 777 {$file_dir}");
        }
        if (!file_exists($setPath)) @touch($setPath);
        $arr = parse_ini_file($setPath, true);
        if(empty($arr)){
            $md = $this->get_system_filetype();
            $app["app"] = array("app_beta" => "1", "app_install_start" => 1,"app_storage_path" => $md);
            $this->_fileEdit($setPath, $app);
        }
        $arr = parse_ini_file($setPath, true);
        show_json($arr["app"]);
    }

    public function get_system_filetype(){
        $s = new storage();
        $mds = $s->_dm_tree();
        $md = "";
        if(count($mds) > 0){
            foreach($mds as $key => $item){
                $md = strstr($item["name"], 'md') ? $key : "lv".substr($item["name"], -1, 1);
                if($md) break;
            }
        }
        return $md;
    }

    /**
     * app自动应设置
     * */
    public function set_app_conf(){
        $setPath = "/etc/store/app.conf";
        $arr = parse_ini_file($setPath, true);
        $opt = $this->in['opt'];
        /*if(isset($opt["update"])){
            $arr["app"]["app_auto_update"] = $opt["update"];
        }*/
        if(isset($opt["beta"])){
            $arr["app"]["app_beta"] = $opt["beta"];
        }
        if(isset($opt["installStart"])){
            $arr["app"]["app_install_start"] = $opt["installStart"];
        }
        if(isset($opt["path"])){
            set_time_limit(0);
            $arr["app"]["app_storage_path"] = $opt["path"];
            $pos = substr($opt["path"], -1);
            $this->_backexec("/etc/tos/scripts/app_change_position /mnt/md{$pos} >/dev/null &");
            sleep(5);
        }
        $this->_fileEdit($setPath, $arr);
        show_json($this->L['success']);
    }

    /**
     * 获取APP端口
     * */
    public function get_app_port(){
        $id = $this->in['id'];
        $path = "/etc/init.d/{$id}";
        if(file_exists($path)){
            $port = @system("{$path} stop >/dev/null");
        }
        show_json($port);
    }

    /**
     * app配置写入
     * */
    private function _fileEdit($path, $array) {
        $file = fopen($path, "wb");
        if (flock($file, LOCK_EX)) {
            foreach ($array as $key => $v) {
                fwrite($file, "[{$key}]\n");
                foreach ($v as $item => $v1) {
                    fwrite($file, "\t{$item} = {$v1}\n");
                }
                fwrite($file, "\n");
            }
            flock($file, LOCK_UN);
        }
        fclose($file);
    }
    /**
     * 重置app
     * */
    public function reset_app() {
        $id = $this->in['id'];
        $path = "/etc/init.d/{$id}";
        if(file_exists($path)){
            @system("{$path} reset >/dev/null");
            show_json($this->L['success'], true);
        }
        show_json($this->L['error'], false);
    }

    /**
     * 存放已安装的 app
     * */
    public function app_installed() {
        $this->init_sql();
        $dataList= $this->in['dataList'];
        $file_dir = "/etc/store/installed";
        if (!file_exists($file_dir)) {
            @system("mkdir -p {$file_dir}");
            @system("chmod -R 777 {$file_dir}");
        }
        $list = self::$netfile->get();
        $list = array_reverse($list);
        foreach ($list as $id => &$item){
            if(!empty($item['category'])){
                $type = $item['category'];
                $item['category_name'] = $GLOBALS['root']['app'][$type];
            }
        }
        foreach ($list as $key => $item){
            foreach ($dataList as $k => $i){
                if($key != $i) continue;
                $filePath = "{$file_dir}/{$key}";
                @touch($filePath);
                $data = json_encode($item);
                @file_put_contents($filePath, $data);
                //$this->_exec("echo '{$data}'> {$filePath}");
            }
        }
        show_json($this->L['success'], true);
    }
    /**
     * 获取已安装的 app
     * */
    public function get_installed_app() {
        $this->init_sql();
        $path = "/etc/store/installed/";
        $appSafetyMod = $this->_exec("cat /etc/store/forbidden_list.conf", "array");
        $absoluteSafe = $this->_exec("cat /etc/firewalld/absolute_safe.conf");
        $absoluteSafe = explode("=", $absoluteSafe);
        $lsit = $this->get_dir_file_conetent($path);
        $lsitFile = [];
        foreach ($lsit as $id => $data ) {
            if($absoluteSafe[1] == 1 && is_array($appSafetyMod) && in_array($data["id"], $appSafetyMod)) continue;
            $lsitFile[$id] = $data;
        }
        show_json($lsitFile, true, $this->L['success']);
    }

    function get_dir_file_conetent($path){
        $list = array();
        if(!file_exists($path)) return false;
        $handle = opendir($path);
        if($handle === false)  return false;
        while(($files = readdir($handle)) !== false){
            if($files == "." || $files == "..") continue;
            $filePath = $path."/{$files}";
            if(!file_exists($filePath)) continue;
            //逐行读取数据
            $file = fopen($filePath,"r");
            while(!feof($file)){
                $data = json_decode(fgets($file), true);
                if(count($data) > 0) $list[$data["id"]] = $data;
            }
            fclose($file);
        }
        closedir($handle);
        return $list;
    }

    /**
     * 应用运行状态
     * */
    public function app_inspection_process(){
        $posInstall = $this->_exec("ps -ef | grep -v grep | grep \"pkgs/.*/install\" | awk '{print $9}'", "array");
        $posRemove = $this->_exec("ps -ef | grep -v grep | grep \"apt-get .*remove\" | awk '{print $11}'", "array");
        $result = array();
        $result["install"] = "";
        if(count($posInstall)>0){
            $res = array();
            foreach($posInstall as $key => $item){
                $item = explode("/", $item);
                $res[$key] = $item[ count($item) - 2 ];
            }
            $result["install"] = $res;
        }
        $result["remove"] = "";
        if(count($posRemove)>0){
            $res = array();
            foreach($posRemove as $key => $item){
                $item = explode("/", $item);
                $res[$key] = $item[ count($item) - 1 ];
            }
            $result["remove"] = $res;
        }
        show_json($result);
    }

    public function get_url_title(){
        $html = curl_get_contents($this->in['url']);
        $result = match($html,"<title>(.*)<\/title>");
        if (strlen($result)>50) {
            $result = mb_substr($result,0,50,'utf-8');
        }
        if (!$result || strlen($result) == 0) {
            $result = $this->in['url'];
            $result = str_replace(array('http://','&','/'),array('','@','-'), $result);
        }
        show_json($result);
    }

    private function _init(){
        return  json_decode(rawurldecode($this->in['data']));
    }
}
